@@ -8,6 +8,7 @@ set -e
 
 sysconfdir="$1"
 bindir="$2"
+udevrulesdir="$3"
 prefix="${MESON_INSTALL_DESTDIR_PREFIX}"
 
 chown root:root "${prefix}/${bindir}/fusermount3"
@@ -19,7 +20,7 @@ if test ! -e "${DESTDIR}/dev/fuse"; then
 fi
 
 install -D -m 644 "${MESON_SOURCE_ROOT}/util/udev.rules" \
-        "${prefix}/lib/udev/rules.d/99-fuse3.rules"
+        "${DESTDIR}/${udevrulesdir}/udev/rules.d/99-fuse3.rules"
 
 install -D -m 755 "${MESON_SOURCE_ROOT}/util/init_script" \
         "${DESTDIR}/etc/init.d/fuse3"
